<?php
/*
 * Google Meet for LatePoint
 * Copyright (c) 2022-2023 TechXela (https://codecanyon.net/user/tech-xela). All Rights Reserved.
 *
 * LICENSE
 * -------
 * This software is furnished under license(s) and may be used and copied only in accordance with the terms of such
 * license(s) along with the inclusion of the above copyright notice. If you purchased this software through CodeCanyon,
 * please read the full license(s) at: https://codecanyon.net/licenses/standard
 */

if ( ! class_exists( 'TechXelaLatePointGoogleMeetBookingHelper' ) ) :

	final class TechXelaLatePointGoogleMeetBookingHelper {
		use \TechXela\LatePointAddons\Traits\Meetings\IsMeetingsBookingHelper;

        public static $meetingSystemCode;
		public static $meetingSystemName;

		public static $settingsHelperClass = TechXelaLatePointGoogleMeetSettingsHelper::class;
		public static $serviceHelperClass = TechXelaLatePointGoogleMeetServiceHelper::class;
		public static $meetingHelperClass = TechXelaLatePointGoogleMeetMeetingHelper::class;

		public static function bookingQuickFormAfter( OsBookingModel $booking ) {
			if ( $booking->exists() ) {
				$meetingId  = TechXelaLatePointGoogleMeetMeetingHelper::getMeetingIdForBookingId( $booking->id );
				$meetingUrl = TechXelaLatePointGoogleMeetMeetingHelper::getMeetingUrlForBookingId( $booking->id );

				if ( $meetingId && $meetingUrl ) {
					?>
                    <div class="os-form-sub-header">
                        <h3><?php esc_html_tx_e( 'Google Meet Info', 'google-meet-latepoint' ); ?></h3></div>
                    <div class="tx-meetings-info-link">
                        <img src="<?php esc_attr_e( \TechXelaLatePointMeetingsSettingsHelper::getMeetingSystemIconUrl( TechXelaLatePointGoogleMeet::class ) ); ?>"
                             alt="Google Meet">
                        <div class="meeting-info">
                            <div class="meeting-id">
                                <span><?php esc_html_tx_e( 'ID', 'latepoint' ); ?>: </span><strong><?php esc_html_e( $meetingId ); ?></strong>
                            </div>
                        </div>
                        <a href="<?php esc_attr_e( $meetingUrl ); ?>"
                           target="_blank"><?php esc_html_tx_e( 'Join', 'google-meet-latepoint' ); ?></a>
                    </div>
					<?php
				}
			}
		}

		protected static function getBookingVars(): array {
			return [
				self::prefixVarNeedle( 'meeting_url' ) => esc_html_tx__( 'Meeting URL', 'google-meet-latepoint' ),
				self::prefixVarNeedle( 'meeting_id' )  => esc_html_tx__( 'Meeting ID', 'google-meet-latepoint' ),
			];
		}

		protected static function getBookingVarsReplacements( $text, $needles, $booking ): array {
			return [
				TechXelaLatePointGoogleMeetMeetingHelper::getMeetingUrlForBookingId( $booking->id ),
				TechXelaLatePointGoogleMeetMeetingHelper::getMeetingIdForBookingId( $booking->id ),
			];
		}
	}
endif;