<?php
/*
 * Google Meet for LatePoint
 * Copyright (c) 2022-2023 TechXela (https://codecanyon.net/user/tech-xela). All Rights Reserved.
 *
 * LICENSE
 * -------
 * This software is furnished under license(s) and may be used and copied only in accordance with the terms of such
 * license(s) along with the inclusion of the above copyright notice. If you purchased this software through CodeCanyon,
 * please read the full license(s) at: https://codecanyon.net/licenses/standard
 */

if ( ! class_exists( 'TechXelaLatePointGoogleMeetServiceHelper' ) ) :

	final class TechXelaLatePointGoogleMeetServiceHelper {
		use \TechXela\LatePointAddons\Traits\Meetings\IsMeetingsServiceHelper;

		public static $meetingSystemCode;
		public static $meetingSystemName;

		public static $meetingSystemViewsAbsPath = TECHXELA_LATEPOINT_GMEET_VIEWS_ABSPATH;

		public static $settingsHelperClass = TechXelaLatePointGoogleMeetSettingsHelper::class;

		public static function isEnabledForService( $service_id ): bool {
			return ( OsMetaHelper::get_service_meta_by_key( 'enable_' . self::$meetingSystemCode, $service_id ) == 'on' );
		}

		public static function groupBookingsEnabled( $serviceId ): bool {
			return ( OsMetaHelper::get_service_meta_by_key( self::$meetingSystemCode . '_group_bookings_enabled', $serviceId ) == 'on' );
		}

		public static function guestsCanInviteOthers( $serviceId ): bool {
			return ( OsMetaHelper::get_service_meta_by_key( 'gmeet_guests_can_invite_others', $serviceId ) == 'on' );
		}

		public static function guestsCanSeeOtherGuests( $serviceId ): bool {
			return ( OsMetaHelper::get_service_meta_by_key( 'gmeet_guests_can_see_other_guests', $serviceId ) == 'on' );
		}

		public static function isPhoneEntryPointEnabled( $serviceId ): bool {
			return self::getEntryPointData( $serviceId, 'phone', 'enabled' ) == 'on';
		}

		public static function getPhoneEntryPointLabel( $serviceId ) {
			return self::getEntryPointData( $serviceId, 'phone', 'label' );
		}

		public static function getPhoneEntryPointUri( $serviceId ) {
			return str_replace( 'tel:', '', self::getEntryPointData( $serviceId, 'phone', 'uri' ) );
		}

		public static function getPhoneEntryPointPin( $serviceId ) {
			return self::getEntryPointData( $serviceId, 'phone', 'pin' );
		}

		public static function isSipEntryPointEnabled( $serviceId ): bool {
			return self::getEntryPointData( $serviceId, 'sip', 'enabled' ) == 'on';
		}

		public static function getSipEntryPointUri( $serviceId ) {
			return str_replace( 'sip:', '', self::getEntryPointData( $serviceId, 'sip', 'uri' ) );
		}

		public static function getSipEntryPointPin( $serviceId ) {
			return self::getEntryPointData( $serviceId, 'sip', 'pin' );
		}

		public static function isMoreEntryPointEnabled( $serviceId ): bool {
			return self::getEntryPointData( $serviceId, 'more', 'enabled' ) == 'on';
		}

		public static function getMoreEntryPointUri( $serviceId ) {
			return self::getEntryPointData( $serviceId, 'more', 'uri' );
		}

		public static function getEntryPointLabel( $serviceId, $entryPoint ) {
			return self::getEntryPointData( $serviceId, $entryPoint, 'label' );
		}

		public static function getEntryPointUri( $serviceId, $entryPoint ) {
			return self::getEntryPointData( $serviceId, $entryPoint, 'uri' );
		}

		public static function getEntryPointPin( $serviceId, $entryPoint ) {
			return self::getEntryPointData( $serviceId, $entryPoint, 'pin' );
		}

		private static function getEntryPointData( $serviceId, $entryPoint, $dataKey ) {
			return OsMetaHelper::get_service_meta_by_key( "gmeet_entry_points_{$entryPoint}_$dataKey", $serviceId );
		}
	}

endif;