<?php 
class OsCouponHelper {

	public static function calculate_coupon_discount($booking){
		$booking->coupon_discount = OsMoneyHelper::pad_to_db_format($booking->full_amount_to_charge(false, false) - $booking->full_amount_to_charge(true, false));
	}

   public static function get_payment_total_info_with_coupon_html($html, $booking){
      $coupon_applied = ($booking->coupon_code) ? 'coupon-is-applied' : '';
      $regular_price = $booking->full_amount_to_charge(false, false);
      $deposit_price = $booking->deposit_amount_to_charge(false);

      $discounted_price_formatted = $booking->formatted_full_price(true, true);
      $regular_price_formatted = $booking->formatted_full_price(false, true);
      $price_html = ($discounted_price_formatted != $regular_price_formatted) ? '<span>'.$regular_price_formatted.'</span>'.$discounted_price_formatted : $regular_price_formatted;
      $deposit_price_formatted = $booking->formatted_deposit_price();
      $price_portion_html = '';

      if($regular_price > 0) $price_portion_html.= '<div class="payment-total-price-w"><span>'.__('Total booking price: ', 'latepoint-coupons').'</span><span class="lp-price-value">'.$price_html.'</span></div>';
      if($deposit_price > 0) $price_portion_html.= '<div class="payment-deposit-price-w"><span>'.__('Deposit Amount: ', 'latepoint-coupons').'</span><span class="lp-price-value">'.$deposit_price_formatted.'</span></div>';

      $payment_portion = (OsBookingHelper::get_default_payment_portion_type($booking) == LATEPOINT_PAYMENT_PORTION_DEPOSIT) ? ' paying-deposit ' : '';
      $html = '<div class="payment-total-info '.$coupon_applied.$payment_portion.'">
                  '.$price_portion_html.'
                  <div class="coupon-code-trigger-w"><a href="#">'.__('Have a coupon code?', 'latepoint-coupons').'</a></div>
                  <div class="coupon-code-input-w">
                    <input type="text" name="coupon_code" class="coupon-code-input" data-route="'. OsRouterHelper::build_route_name('coupons', 'apply').'" placeholder="'.__('Enter Code...', 'latepoint-coupons').'">
                    <div class="coupon-code-input-submit">'.__('OK', 'latepoint-coupons').'</div>
                    <input type="hidden" name="booking[coupon_code]" value="'.$booking->coupon_code.'"/>
                  </div>
                  <div class="applied-coupon-code">
                    <span class="coupon-code-self">'. $booking->coupon_code.'</span>
                    <span class="coupon-code-clear"><i class="latepoint-icon latepoint-icon-common-01"></i></span>
                  </div>
                </div>';
      return $html;
   }

	public static function get_coupon_by_code($coupon_code){
		$coupon = new OsCouponModel();
    $coupon = $coupon->where(['code' => $coupon_code, 'status' => LATEPOINT_COUPON_STATUS_ACTIVE])->set_limit(1)->get_results_as_models();
		return $coupon;
	}

	public static function apply_coupons_on_amount($amount, $booking_object, $apply_coupons, $apply_taxes){
    if(!$apply_coupons) return $amount;
		if(empty($booking_object->coupon_code)) return $amount;
		if(!is_wp_error(self::is_coupon_code_valid($booking_object->coupon_code, $booking_object))){
			$coupon = self::get_coupon_by_code($booking_object->coupon_code);
			if($coupon->discount_type == 'percent'){
				if($coupon->discount_value) $amount = round($amount - ($amount * ($coupon->discount_value / 100)), 2);
			}elseif($coupon->discount_type == 'fixed'){
				if($coupon->discount_value) $amount = $amount - $coupon->discount_value;
			}
		}
		if($amount < 0) $amount = 0;
		return $amount;
	}


	public static function is_coupon_code_valid($coupon_code, $booking_object = false){
		if(empty($coupon_code)) return __('Invalid Coupon Code', 'latepoint-coupons');
		$valid = true;
		$reason = '';
    $coupon = new OsCouponModel();
    $coupon = $coupon->where(['code' => $coupon_code, 'status' => LATEPOINT_COUPON_STATUS_ACTIVE])->set_limit(1)->get_results_as_models();

   	if($coupon){
      // Customer id
      if($coupon->get_rule('customer_ids')){
        $allowed_customer_ids = explode(',', str_replace(' ', '', $coupon->get_rule('customer_ids')));
        if(!empty($allowed_customer_ids) && !in_array($booking_object->customer_id, $allowed_customer_ids)){
          $valid = false;
          $reason = __('This coupon code is invalid or has expired', 'latepoint-coupons');
        }
      }
      // Agent id
      if($coupon->get_rule('agent_ids')){
        $allowed_agent_ids = explode(',', str_replace(' ', '', $coupon->get_rule('agent_ids')));
        if(!empty($allowed_agent_ids) && !in_array($booking_object->agent_id, $allowed_agent_ids)){
          $valid = false;
          $reason = __('This coupon code is invalid or has expired', 'latepoint-coupons');
        }
      }
   		// Service id
			if($coupon->get_rule('service_ids')){
        $allowed_service_ids = explode(',', str_replace(' ', '', $coupon->get_rule('service_ids')));
        if(!empty($allowed_service_ids) && !in_array($booking_object->service_id, $allowed_service_ids)){
   				$valid = false;
   				$reason = __('This coupon code is invalid or has expired', 'latepoint-coupons');
   			}
			}
      // limit
      if($coupon->get_rule('limit_total')){
        $booking = new OsBookingModel();
				if($booking_object && $booking_object->id) $booking->where(['id !=' => $booking_object->id]);
        $total_orders = $booking->where(['coupon_code' => $coupon->code])->should_not_be_cancelled()->count();
        if($total_orders >= $coupon->get_rule('limit_total')){
          $valid = false;
          $reason = sprintf(__('This coupon can only be used %d times', 'latepoint-coupons'), $coupon->get_rule('limit_total'));
        }
      }
   		// per customer limit
   		if($booking_object && $booking_object->customer_id && $coupon->get_rule('limit_per_customer')){
   			$booking = new OsBookingModel();
				if($booking_object && $booking_object->id) $booking->where(['id !=' => $booking_object->id]);
   			$total_orders = $booking->where(['coupon_code' => $coupon->code, 'customer_id' => $booking_object->customer_id])->should_not_be_cancelled()->count();
   			if($total_orders >= $coupon->get_rule('limit_per_customer')){
   				$valid = false;
   				$reason = sprintf(__('This coupon is limited to %d uses per customer', 'latepoint-coupons'), $coupon->get_rule('limit_per_customer'));
   			}
   		}
   		// min orders
			if($booking_object && $booking_object->customer_id && $coupon->get_rule('orders_more')){
   			$booking = new OsBookingModel();
				if($booking_object && $booking_object->id) $booking->where(['id !=' => $booking_object->id]);
   			$total_orders = $booking->where(['customer_id' => $booking_object->customer_id])->should_not_be_cancelled()->count();
   			if($total_orders <= $coupon->get_rule('orders_more')){
   				$valid = false;
   				$reason = sprintf(__('You need to have at least %d orders to use this coupon', 'latepoint-coupons'), $coupon->get_rule('orders_more'));
   			}
			}
   		// max orders
			if($booking_object && $booking_object->customer_id && $coupon->get_rule('orders_less')){
   			$booking = new OsBookingModel();
				if($booking_object && $booking_object->id) $booking->where(['id !=' => $booking_object->id]);
   			$total_orders = $booking->where(['customer_id' => $booking_object->customer_id])->should_not_be_cancelled()->count();
   			if($total_orders >= $coupon->get_rule('orders_less')){
   				$valid = false;
   				if($coupon->get_rule('orders_less') == 1){
	   				$reason = __('This coupon can only be applied on your first order', 'latepoint-coupons');
   				}else{
	   				$reason = sprintf(__('This coupon can only be applied on your first %d orders', 'latepoint-coupons'), $coupon->get_rule('orders_less'));
   				}
   			}
			}
   	}else{
   		$valid = false;
   		$reason = __('This coupon code is invalid or has expired', 'latepoint-coupons');
   	}
   	if($valid){
   		return true;
   	}else{
	   	return new WP_Error('coupon_invalid', $reason);
   	}
	}
}
