<?php
/**
 * Plugin Name: LatePoint Addon - Locations
 * Plugin URI:  https://latepoint.com/
 * Description: LatePoint addon for locations
 * Version:     1.1.2
 * Author:      LatePoint
 * Author URI:  https://latepoint.com/
 * Text Domain: latepoint-locations
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

// If no LatePoint class exists - exit, because LatePoint plugin is required for this addon

if ( ! class_exists( 'LatePointAddonLocations' ) ) :

/**
 * Main Addon Class.
 *
 */

class LatePointAddonLocations {

  /**
   * Addon version.
   *
   */
  public $version = '1.1.2';
  public $db_version = '1.0.1';
  public $addon_name = 'latepoint-locations';




  /**
   * LatePoint Constructor.
   */
  public function __construct() {
    $this->define_constants();
    $this->init_hooks();
  }

  /**
   * Define LatePoint Constants.
   */
  public function define_constants() {
    $this->define( 'LATEPOINT_ADDON_LOCATIONS_ABSPATH', dirname( __FILE__ ) . '/' );
    $this->define( 'LATEPOINT_ADDON_LOCATIONS_LIB_ABSPATH', LATEPOINT_ADDON_LOCATIONS_ABSPATH . 'lib/' );
    $this->define( 'LATEPOINT_ADDON_LOCATIONS_VIEWS_ABSPATH', LATEPOINT_ADDON_LOCATIONS_LIB_ABSPATH . 'views/' );
  }


  public static function public_stylesheets() {
    return plugin_dir_url( __FILE__ ) . 'public/stylesheets/';
  }

  /**
   * Define constant if not already set.
   *
   */
  public function define( $name, $value ) {
    if ( ! defined( $name ) ) {
      define( $name, $value );
    }
  }

  /**
   * Include required core files used in admin and on the frontend.
   */
  public function includes() {

    // CONTROLLERS
    include_once( dirname( __FILE__ ) . '/lib/controllers/locations_controller.php' );
    include_once( dirname( __FILE__ ) . '/lib/controllers/location_categories_controller.php' );

    // HELPERS

    // MODELS

  }

  public function init_hooks(){
    add_action('latepoint_includes', [$this, 'includes']);

    add_action( 'init', array( $this, 'init' ), 0 );

    add_action('latepoint_wp_enqueue_scripts', [$this, 'load_front_scripts_and_styles']);
    add_action('latepoint_admin_enqueue_scripts', [$this, 'load_admin_scripts_and_styles']);

    add_action( 'latepoint_settings_steps_after', array( $this, 'add_location_categories_setting' ) );

    add_action('latepoint_booking_summary_before_price_breakdown',[$this, 'add_location_information']);


		// add location data to booking model data_vars
    add_filter('latepoint_model_view_as_data', [$this, 'add_location_data_vars_to_booking'], 10, 2);

    add_filter('latepoint_side_menu', [$this, 'add_menu_links']);
    add_filter('latepoint_installed_addons', [$this, 'register_addon']);
    add_filter('latepoint_locations_addon_installed', [$this, 'show_locations']);
    add_filter('latepoint_custom_field_condition_properties', [$this, 'add_custom_field_condition_properties']);
    add_filter('latepoint_available_values_for_condition_property', [$this, 'add_values_for_condition_property'], 10, 2);
    add_filter('latepoint_model_options_for_multi_select', [$this, 'add_options_for_multi_select'], 10, 2);
    add_filter('latepoint_process_event_trigger_condition_properties', [$this, 'add_process_event_condition_properties'], 10, 2);
    add_filter('latepoint_available_values_for_process_event_condition_property', [$this, 'add_values_for_process_event_condition_properties'], 10, 2);

    add_filter('latepoint_bookings_data_for_csv_export', [$this, 'add_location_to_bookings_data_for_csv'], 11, 2);
    add_filter('latepoint_booking_row_for_csv_export', [$this, 'add_location_to_booking_row_for_csv'], 11, 3);

    add_filter('latepoint_webhook_variables_for_new_booking', [$this, 'add_booking_location_to_webhook'], 10, 2);
		add_filter('latepoint_capabilities_for_controllers', [$this, 'set_capabilities']);

    register_activation_hook(__FILE__, [$this, 'on_activate']);
    register_deactivation_hook(__FILE__, [$this, 'on_deactivate']);


  }

  /**
   * Init LatePoint when WordPress Initialises.
   */
  public function init() {
    // Set up localisation.
    $this->load_plugin_textdomain();
  }


	public function set_capabilities($capabilities) {
		$capabilities['OsLocationsController'] = [
			'default' => ['location__edit'],
			'per_action' => [
				'edit_form' => ['location__view'],
				'index' => ['location__view'],
				'new_form' => ['location__create'],
				'destroy' => ['location__delete'],
				'create' => ['location__create'],
			]
		];
		return $capabilities;
	}

	public function add_location_data_vars_to_booking(array $data, OsModel $model): array{
		if(is_a($model, 'OsBookingModel')){
			$data['location'] = $model->location->get_data_vars();
		}
		return $data;
	}

  public function add_booking_location_to_webhook($vars, $booking){
		if($booking->location_id) $vars['location'] = $booking->location->name;
    return $vars;
  }

	public function add_process_event_condition_properties($properties, $event_type){
		switch ($event_type){
			case 'booking_created':
				$properties['booking__location_id'] = __('Location', 'latepoint');
				break;
			case 'booking_updated':
				$properties['old__booking__location_id'] = __('Previous Location', 'latepoint');
				$properties['booking__location_id'] = __('Location', 'latepoint');
				break;
		}
		return $properties;
	}

  public function add_menu_links($menus){
    for($i=0; $i<count($menus); $i++){
      if(isset($menus[$i]['id']) && $menus[$i]['id'] == 'locations'){
        $menus[$i] = [ 'id' => 'locations', 'label' => __( 'Locations', 'latepoint' ), 'icon' => 'latepoint-icon latepoint-icon-map-pin', 'link' => OsRouterHelper::build_link(['locations', 'index']),
          'children' => [
                          ['label' => __( 'All Locations', 'latepoint' ), 'icon' => '', 'link' => OsRouterHelper::build_link(['locations', 'index'])],
                          ['label' => __( 'Categories', 'latepoint' ), 'icon' => '', 'link' => OsRouterHelper::build_link(['location_categories', 'index'])],
        ]];
      }
    }
    return $menus;
  }

	public function add_values_for_process_event_condition_properties($values, $property){
    switch($property){
      case 'booking__location_id':
        $values = OsFormHelper::model_options_for_multi_select($property);
        break;
    }
		return $values;
	}

  public function add_options_for_multi_select($options, $model_name){
    if($model_name == 'location' || $model_name == 'OsLocationModel'){
      $location = new OsLocationModel;
      $locations = $location->get_results_as_models();
      if($locations){
        foreach($locations as $location){
          $options[] = ['value' => $location->id, 'label' => $location->name];
        }
      }
    }
    return $options;
  }

  public function add_values_for_condition_property($values, $property){
    if($property == 'location'){
      $values = OsFormHelper::model_options_for_multi_select('location');
    }
    return $values;
  }


  public function add_custom_field_condition_properties($properties){
    $properties['location'] = __('Location', 'latepoint-locations');
    return $properties;
  }

  public function add_location_information($booking){
    if($booking->location_id && $booking->location_id != LATEPOINT_ANY_LOCATION){
      $locations = new OsLocationModel();
      $location_ids = $locations->select('id')->should_be_active()->get_results(ARRAY_A);
			$location = $booking->get_location();
			// only show location if there are multiple in database or is location has a full address set
      if((is_array($location_ids) && count($location_ids) > 1) || !empty($location->full_address)){ ?>
			  <div class="summary-box sb-location">
					<div class="summary-box-heading">
						<div class="sbh-item"><?php _e('Location', 'latepoint') ?></div>
						<div class="sbh-line"></div>
					</div>
				  <div class="summary-box-content with-media">
					  <div class="os-location-image"></div>
					  <div class="sbc-content-i">
						  <div class="sbc-main-item">
							  <?php
							  $location = $booking->get_location();
							  echo $location->name;
								if(!empty($location->full_address)) echo ' <a href="'.$location->get_google_maps_link().'" target="_blank"><i class="latepoint-icon latepoint-icon-external-link"></i></a>';
								?>
						  </div>
						  <?php if($location->full_address) echo '<div class="sbc-sub-item">'.$location->full_address.'</div>'; ?>
					  </div>
				  </div>
			  </div><?php
      }
    }
  }

  public function add_location_to_booking_row_for_csv($booking_row, $booking, $params = []){
    $booking_row[] = $booking->location->name;
    return $booking_row;
  }


  public function add_location_to_bookings_data_for_csv($bookings_data, $params = []){
    $bookings_data[0][] = __('Location', 'latepoint-locations');
    return $bookings_data;
  }



  public function add_location_categories_setting(){
    echo OsFormHelper::toggler_field('settings[steps_show_location_categories]', __('Show location categories', 'latepoint-locations'), OsSettingsHelper::is_on('steps_show_location_categories'), false, false, ['sub_label' => __('If turned on, locations will be displayed in categories', 'latepoint-locations')]);
  }


  public function show_locations($show_locations){
    return true;
  }


  public function load_plugin_textdomain() {
    load_plugin_textdomain('latepoint-locations', false, dirname(plugin_basename(__FILE__)) . '/languages');
  }

  public function on_deactivate(){
  }

  public function on_activate(){
    if(class_exists('OsDatabaseHelper')) OsDatabaseHelper::check_db_version_for_addons();
    do_action('latepoint_on_addon_activate', $this->addon_name, $this->version);
  }

  public function register_addon($installed_addons){
    $installed_addons[] = ['name' => $this->addon_name, 'db_version' => $this->db_version, 'version' => $this->version];
    return $installed_addons;
  }


  public function load_front_scripts_and_styles(){
    // Stylesheets

    // Javascripts

  }

  public function load_admin_scripts_and_styles($localized_vars){

    // Stylesheets
    wp_enqueue_style( 'latepoint-locations-admin', $this->public_stylesheets() . 'latepoint-locations-admin.css', false, $this->version );

    // Javascripts

  }




}

endif;
if ( in_array( 'latepoint/latepoint.php', get_option( 'active_plugins', array() ) )  || array_key_exists('latepoint/latepoint.php', get_site_option('active_sitewide_plugins', array())) ) {
	$LATEPOINT_ADDON_LOCATIONS = new LatePointAddonLocations();
}
$latepoint_session_salt = 'NmQ3OTBhMTItYjY5ZS00ZGFlLTg5ODMtNTlkYzVjZTM4MTVi';
