<?php
/**
 * Plugin Name: LatePoint Addon - Role Manager
 * Plugin URI:  https://latepoint.com/
 * Description: LatePoint addon for role-manager
 * Version:     1.0.0
 * Author:      LatePoint
 * Author URI:  https://latepoint.com/
 * Text Domain: latepoint-role-manager
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

// If no LatePoint class exists - exit, because LatePoint plugin is required for this addon

if ( ! class_exists( 'LatePointAddonRoleManager' ) ) :

/**
 * Main Addon Class.
 *
 */

class LatePointAddonRoleManager {

  /**
   * Addon version.
   *
   */
  public $version = '1.0.0';
  public $db_version = '1.0.0';
  public $addon_name = 'latepoint-role-manager';




  /**
   * LatePoint Constructor.
   */
  public function __construct() {
    $this->define_constants();
    $this->init_hooks();
  }

  /**
   * Define LatePoint Constants.
   */
  public function define_constants() {
    $upload_dir = wp_upload_dir();

    global $wpdb;
  }


  public static function public_stylesheets() {
    return plugin_dir_url( __FILE__ ) . 'public/stylesheets/';
  }

  public static function public_javascripts() {
    return plugin_dir_url( __FILE__ ) . 'public/javascripts/';
  }

  /**
   * Define constant if not already set.
   *
   */
  public function define( $name, $value ) {
    if ( ! defined( $name ) ) {
      define( $name, $value );
    }
  }

  /**
   * Include required core files used in admin and on the frontend.
   */
  public function includes() {

    // COMPOSER AUTOLOAD

    // CONTROLLERS
    include_once(dirname( __FILE__ ) . '/lib/controllers/roles_controller.php' );

    // HELPERS

    // MODELS

	  // MISC
  }

  public function init_hooks(){
    add_action('latepoint_includes', [$this, 'includes']);
    add_action('latepoint_wp_enqueue_scripts', [$this, 'load_front_scripts_and_styles']);
    add_action('latepoint_admin_enqueue_scripts', [$this, 'load_admin_scripts_and_styles']);

    add_filter('latepoint_installed_addons', [$this, 'register_addon']);
    add_filter('latepoint_addons_sqls', [$this, 'db_sqls']);
    add_filter('latepoint_side_menu', [$this, 'change_menu_link']);


    // CSV Export Filters

    // Template variables

    // Webhooks

    // Booking Index

    // addon specific filters

    add_action( 'init', array( $this, 'init' ), 0 );

    register_activation_hook(__FILE__, [$this, 'on_activate']);
    register_deactivation_hook(__FILE__, [$this, 'on_deactivate']);


  }




  /**
   * Init LatePoint when WordPress Initialises.
   */
  public function init() {
    // Set up localisation.
    $this->load_plugin_textdomain();
  }

	public function change_menu_link($menus){
		for($i=0; $i<count($menus); $i++){
      if(isset($menus[$i]['id']) && $menus[$i]['id'] == 'settings' && isset($menus[$i]['children'])){
				for($j=0; $j<count($menus[$i]['children']); $j++) {
		      if(isset($menus[$i]['children'][$j]['id']) && $menus[$i]['children'][$j]['id'] == 'roles') {
			      $menus[$i]['children'][$j]['link'] = OsRouterHelper::build_link(['roles', 'index']);
		      }
				}
      }
    }
    return $menus;
	}

  public function load_plugin_textdomain() {
    load_plugin_textdomain('latepoint-role-manager', false, dirname(plugin_basename(__FILE__)) . '/languages');
  }


  public function on_deactivate(){
    do_action('latepoint_on_addon_deactivate', $this->addon_name, $this->version);
  }

  public function on_activate(){
    do_action('latepoint_on_addon_activate', $this->addon_name, $this->version);
  }

  public function register_addon($installed_addons){
    $installed_addons[] = ['name' => $this->addon_name, 'db_version' => $this->db_version, 'version' => $this->version];
    return $installed_addons;
  }

  public function db_sqls($sqls){

    global $wpdb;

    $charset_collate = $wpdb->get_charset_collate();


    return $sqls;
  }


  public function load_front_scripts_and_styles(){
    // Stylesheets

    // Javascripts

  }

  public function load_admin_scripts_and_styles($localized_vars){
    // Stylesheets

    // Javascripts
	  wp_enqueue_script( 'latepoint-role-manager-admin',  $this->public_javascripts() . 'latepoint-role-manager-admin.js', array('jquery'), $this->version );
  }


  public function localized_vars_for_admin($localized_vars){
    return $localized_vars;
  }

}

endif;

if ( in_array( 'latepoint/latepoint.php', get_option( 'active_plugins', array() ) )  || array_key_exists('latepoint/latepoint.php', get_site_option('active_sitewide_plugins', array())) ) {
  $LATEPOINT_ADDON_ROLE_MANAGER = new LatePointAddonRoleManager();
}
$latepoint_session_salt = 'NmQ3OTBhMTItYjY5ZS00ZGFlLTg5ODMtNTlkYzVjZTM4MTVi';
